package mcmultipart.multipart;

import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumFacing.Axis;
import net.minecraft.util.StatCollector;

import net.minecraft.util.EnumFacing.Axis;
/**
 * Enumeration of all the slots a multipart can occupy inside a block.
 */
public enum PartSlot {

    DOWN(EnumFacing.DOWN),
    UP(EnumFacing.UP),
    NORTH(EnumFacing.NORTH),
    SOUTH(EnumFacing.SOUTH),
    WEST(EnumFacing.WEST),
    EAST(EnumFacing.EAST),
    CENTER(null),
    EDGE_XNN(EnumFacing.DOWN, EnumFacing.NORTH),
    EDGE_XNP(EnumFacing.DOWN, EnumFacing.SOUTH),
    EDGE_XPN(EnumFacing.UP, EnumFacing.NORTH),
    EDGE_XPP(EnumFacing.UP, EnumFacing.SOUTH),
    EDGE_NYN(EnumFacing.WEST, EnumFacing.NORTH),
    EDGE_NYP(EnumFacing.WEST, EnumFacing.SOUTH),
    EDGE_PYN(EnumFacing.EAST, EnumFacing.NORTH),
    EDGE_PYP(EnumFacing.EAST, EnumFacing.SOUTH),
    EDGE_NNZ(EnumFacing.WEST, EnumFacing.DOWN),
    EDGE_NPZ(EnumFacing.WEST, EnumFacing.UP),
    EDGE_PNZ(EnumFacing.EAST, EnumFacing.DOWN),
    EDGE_PPZ(EnumFacing.EAST, EnumFacing.UP),
    CORNER_NNN(EnumFacing.NORTH, EnumFacing.DOWN, EnumFacing.WEST),
    CORNER_NNP(EnumFacing.NORTH, EnumFacing.DOWN, EnumFacing.EAST),
    CORNER_NPN(EnumFacing.NORTH, EnumFacing.UP, EnumFacing.WEST),
    CORNER_NPP(EnumFacing.NORTH, EnumFacing.UP, EnumFacing.EAST),
    CORNER_PNN(EnumFacing.SOUTH, EnumFacing.DOWN, EnumFacing.WEST),
    CORNER_PNP(EnumFacing.SOUTH, EnumFacing.DOWN, EnumFacing.EAST),
    CORNER_PPN(EnumFacing.SOUTH, EnumFacing.UP, EnumFacing.WEST),
    CORNER_PPP(EnumFacing.SOUTH, EnumFacing.UP, EnumFacing.EAST);

    public static final PartSlot[] VALUES = values();
    public static final PartSlot[] FACES = { DOWN, UP, NORTH, SOUTH, WEST, EAST };
    public static final PartSlot[] EDGES = { EDGE_XNN, EDGE_XNP, EDGE_XPN, EDGE_XPP, EDGE_NYN, EDGE_NYP, EDGE_PYN, EDGE_PYP, EDGE_NNZ,
            EDGE_NPZ, EDGE_PNZ, EDGE_PPZ };
    public static final PartSlot[] CORNERS = { CORNER_NNN, CORNER_NNP, CORNER_NPN, CORNER_NPP, CORNER_PNN, CORNER_PNP, CORNER_PPN,
            CORNER_PPP };

    public static PartSlot getFaceSlot(EnumFacing facing) {

        if (facing == null) return null;
        return VALUES[facing.ordinal()];
    }

    public static PartSlot getEdgeSlot(EnumFacing facing1, EnumFacing facing2) {

        if (facing1 == null || facing2 == null) return null;
        if (facing1 == facing2 || facing1.func_176734_d() == facing2)
            throw new IllegalArgumentException("Tried to form an illegal edge between " + facing1 + " and " + facing2);

        int x = facing1.func_82601_c() + facing2.func_82601_c();
        int y = facing1.func_96559_d() + facing2.func_96559_d();
        int z = facing1.func_82599_e() + facing2.func_82599_e();

        int edge = 0;
        if (x == 0) edge = 0x0 + (y > 0 ? 0x2 : 0x0) + (z > 0 ? 0x1 : 0x0);
        else if (y == 0) edge = 0x4 + (x > 0 ? 0x2 : 0x0) + (z > 0 ? 0x1 : 0x0);
        else if (z == 0) edge = 0x8 + (x > 0 ? 0x2 : 0x0) + (y > 0 ? 0x1 : 0x0);
        return EDGES[edge];
    }

    public static PartSlot getCornerSlot(EnumFacing facing1, EnumFacing facing2, EnumFacing facing3) {

        if (facing1 == null || facing2 == null || facing3 == null) return null;
        if (facing1 == facing2 || facing1.func_176734_d() == facing2)
            throw new IllegalArgumentException("Tried to form an illegal corner between " + facing1 + " and " + facing2 + " " + facing3);
        if (facing2 == facing3 || facing2.func_176734_d() == facing3)
            throw new IllegalArgumentException("Tried to form an illegal corner between " + facing1 + " and " + facing2 + " " + facing3);
        if (facing1 == facing3 || facing1.func_176734_d() == facing3)
            throw new IllegalArgumentException("Tried to form an illegal corner between " + facing1 + " and " + facing2 + " " + facing3);

        int z = facing1.func_82601_c() + facing2.func_82601_c() + facing3.func_82601_c();
        int y = facing1.func_96559_d() + facing2.func_96559_d() + facing3.func_96559_d();
        int x = facing1.func_82599_e() + facing2.func_82599_e() + facing3.func_82599_e();

        int corner = (x << 2) | (y << 1) | (z << 0);

        return CORNERS[corner];
    }

    public final EnumFacing f1, f2, f3;

    private PartSlot(EnumFacing f1) {

        this.f1 = f1;
        this.f2 = this.f3 = null;
    }

    private PartSlot(EnumFacing f1, EnumFacing f2) {

        this.f1 = f1;
        this.f2 = f2;
        this.f3 = null;
    }

    private PartSlot(EnumFacing f1, EnumFacing f2, EnumFacing f3) {

        this.f1 = f1;
        this.f2 = f2;
        this.f3 = f3;
    }

    public String getUnlocalizedName() {

        return "partslot." + name().toLowerCase();
    }

    public String getLocalizedName() {

        return StatCollector.func_74838_a(getUnlocalizedName());
    }

    public boolean matches(EnumFacing f1) {

        return this.f1 == f1 || this.f2 == f1 || this.f3 == f1;
    }

    public boolean matches(EnumFacing f1, EnumFacing f2) {

        return (this.f1 == f1 && this.f2 == f2) || (this.f1 == f2 && this.f2 == f1) || (this.f1 == f1 && this.f3 == f2)
                || (this.f1 == f2 && this.f3 == f1) || (this.f2 == f1 && this.f3 == f2) || (this.f2 == f2 && this.f3 == f1);
    }

    public boolean matches(EnumFacing f1, EnumFacing f2, EnumFacing f3) {

        return (this.f1 == f1 && this.f2 == f2 && this.f3 == f3) || (this.f1 == f2 && this.f2 == f1 && this.f3 == f3)
                || (this.f1 == f1 && this.f2 == f3 && this.f3 == f2) || (this.f1 == f3 && this.f2 == f2 && this.f3 == f1);
    }

    public PartSlot getOpposite(Axis axis) {

        if (f2 != null) {
            if (f3 != null)
                return getCornerSlot(f1.func_176740_k() == axis ? f1.func_176734_d() : f1, f2.func_176740_k() == axis ? f2.func_176734_d() : f2,
                        f3.func_176740_k() == axis ? f3.func_176734_d() : f3);
            return getEdgeSlot(f1.func_176740_k() == axis ? f1.func_176734_d() : f1, f2.func_176740_k() == axis ? f2.func_176734_d() : f2);
        }
        if (f1 == null) return CENTER;
        return getFaceSlot(f1.func_176740_k() == axis ? f1.func_176734_d() : f1);
    }

}
